Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(923)

Unified Diff: base/metrics/field_trial.cc

Issue 2530573002: Share field trial allocator on zygote-using Linuxes (Closed)
Patch Set: add posix comments Created 4 years, 1 month ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « base/metrics/field_trial.h ('k') | base/metrics/field_trial_unittest.cc » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: base/metrics/field_trial.cc
diff --git a/base/metrics/field_trial.cc b/base/metrics/field_trial.cc
index 7dc5cb454e55877d7e345bbe13c1a4acca4d2711..1889e8439524b3d8723afeda5fa9c2f932d4d485 100644
--- a/base/metrics/field_trial.cc
+++ b/base/metrics/field_trial.cc
@@ -21,6 +21,17 @@
#include "base/strings/stringprintf.h"
#include "base/strings/utf_string_conversions.h"
+// On systems that use the zygote process to spawn child processes, we must
+// retrieve the correct fd using the mapping in GlobalDescriptors.
+#if defined(OS_POSIX) && !defined(OS_NACL) && !defined(OS_MACOSX) && \
+ !defined(OS_ANDROID)
+#define POSIX_WITH_ZYGOTE 1
+#endif
+
+#if defined(POSIX_WITH_ZYGOTE)
+#include "base/posix/global_descriptors.h"
+#endif
+
namespace base {
namespace {
@@ -233,11 +244,20 @@ HANDLE CreateReadOnlyHandle(FieldTrialList::FieldTrialAllocator* allocator) {
DWORD access = SECTION_MAP_READ | SECTION_QUERY;
HANDLE dst;
if (!::DuplicateHandle(process, src, process, &dst, access, true, 0))
- return nullptr;
+ return kInvalidPlatformHandle;
return dst;
}
#endif
+#if defined(POSIX_WITH_ZYGOTE)
+int CreateReadOnlyHandle(FieldTrialList::FieldTrialAllocator* allocator) {
+ SharedMemoryHandle new_handle;
+ allocator->shared_memory()->ShareReadOnlyToProcess(GetCurrentProcessHandle(),
+ &new_handle);
+ return SharedMemory::GetFdFromSharedMemoryHandle(new_handle);
+}
+#endif
+
} // namespace
// statics
@@ -740,15 +760,12 @@ void FieldTrialList::CreateTrialsFromCommandLine(
const char* field_trial_handle_switch) {
global_->create_trials_from_command_line_called_ = true;
-#if defined(OS_WIN) && !defined(OS_NACL)
if (cmd_line.HasSwitch(field_trial_handle_switch)) {
- std::string arg = cmd_line.GetSwitchValueASCII(field_trial_handle_switch);
- int field_trial_handle = std::stoi(arg);
- HANDLE handle = reinterpret_cast<HANDLE>(field_trial_handle);
- bool result = CreateTrialsFromWindowsHandle(handle);
+ std::string handle_switch =
+ cmd_line.GetSwitchValueASCII(field_trial_handle_switch);
+ bool result = CreateTrialsFromHandleSwitch(handle_switch);
DCHECK(result);
}
-#endif
if (cmd_line.HasSwitch(switches::kForceFieldTrials)) {
bool result = FieldTrialList::CreateTrialsFromString(
@@ -772,6 +789,20 @@ void FieldTrialList::AppendFieldTrialHandleIfNeeded(
}
#endif
+#if defined(OS_POSIX) && !defined(OS_NACL)
+// static
+int FieldTrialList::GetFieldTrialHandle() {
+ if (!global_)
+ return kInvalidPlatformFile;
+ if (kUseSharedMemoryForFieldTrials) {
+ InstantiateFieldTrialAllocatorIfNeeded();
+ if (global_->readonly_allocator_handle_ != kInvalidPlatformFile)
Alexei Svitkine (slow) 2016/11/24 19:04:39 Nit: Seems this if is not needed - since if it's k
lawrencewu 2016/11/24 19:14:57 Done.
+ return global_->readonly_allocator_handle_;
+ }
+ return kInvalidPlatformFile;
+}
+#endif
+
// static
void FieldTrialList::CopyFieldTrialStateToFlags(
const char* field_trial_handle_switch,
@@ -782,25 +813,33 @@ void FieldTrialList::CopyFieldTrialStateToFlags(
if (!global_)
return;
-#if defined(OS_WIN)
+#if defined(OS_WIN) || defined(POSIX_WITH_ZYGOTE)
// Use shared memory to pass the state if the feature is enabled, otherwise
// fallback to passing it via the command line as a string.
if (kUseSharedMemoryForFieldTrials) {
InstantiateFieldTrialAllocatorIfNeeded();
// If the readonly handle didn't get duplicated properly, then fallback to
// original behavior.
- if (!global_->readonly_allocator_handle_) {
+ if (global_->readonly_allocator_handle_ == kInvalidPlatformFile) {
AddForceFieldTrialsFlag(cmd_line);
return;
}
// HANDLE is just typedef'd to void *. We basically cast the handle into an
// int (uintptr_t, to be exact), stringify the int, and pass it as a
// command-line flag. The child process will do the reverse conversions to
// retrieve the handle. See http://stackoverflow.com/a/153077
auto uintptr_handle =
reinterpret_cast<uintptr_t>(global_->readonly_allocator_handle_);
std::string field_trial_handle = std::to_string(uintptr_handle);
+#elif defined(POSIX_WITH_ZYGOTE)
+ // On POSIX we can straight-up cast the handle into a string, since it's
+ // just an int.
+ std::string field_trial_handle =
+ std::to_string(global_->readonly_allocator_handle_);
+#endif
cmd_line->AppendSwitchASCII(field_trial_handle_switch, field_trial_handle);
global_->field_trial_allocator_->UpdateTrackingHistograms();
return;
@@ -893,11 +932,32 @@ size_t FieldTrialList::GetFieldTrialCount() {
return global_->registered_.size();
}
-#if defined(OS_WIN)
// static
-bool FieldTrialList::CreateTrialsFromWindowsHandle(HANDLE handle) {
+bool FieldTrialList::CreateTrialsFromHandleSwitch(
+ const std::string& handle_switch) {
+#if defined(OS_WIN)
+ int field_trial_handle = std::stoi(handle_switch);
+ HANDLE handle = reinterpret_cast<HANDLE>(field_trial_handle);
SharedMemoryHandle shm_handle(handle, GetCurrentProcId());
+ return FieldTrialList::CreateTrialsFromSharedMemoryHandle(shm_handle);
+#endif
+
+#if defined(POSIX_WITH_ZYGOTE)
+ int fd_key = std::stoi(handle_switch);
+ int fd = GlobalDescriptors::GetInstance()->Get(fd_key);
+ SharedMemoryHandle shm_handle(fd, true);
+ return FieldTrialList::CreateTrialsFromSharedMemoryHandle(shm_handle);
+#endif
+
+#if !defined(OS_WIN) && !defined(POSIX_WITH_ZYGOTE)
+ return false;
+#endif
+}
+#if !defined(OS_NACL)
+// static
+bool FieldTrialList::CreateTrialsFromSharedMemoryHandle(
+ SharedMemoryHandle shm_handle) {
// shm gets deleted when it gets out of scope, but that's OK because we need
// it only for the duration of this method.
std::unique_ptr<SharedMemory> shm(new SharedMemory(shm_handle, true));
@@ -987,8 +1047,15 @@ void FieldTrialList::InstantiateFieldTrialAllocatorIfNeeded() {
if (global_->field_trial_allocator_ != nullptr)
return;
+ SharedMemoryCreateOptions options;
+ options.size = kFieldTrialAllocationSize;
+ options.share_read_only = true;
+
std::unique_ptr<SharedMemory> shm(new SharedMemory());
- if (!shm->CreateAndMapAnonymous(kFieldTrialAllocationSize))
+ if (!shm->Create(options))
+ TerminateBecauseOutOfMemory(kFieldTrialAllocationSize);
+
+ if (!shm->Map(kFieldTrialAllocationSize))
TerminateBecauseOutOfMemory(kFieldTrialAllocationSize);
global_->field_trial_allocator_.reset(
@@ -1000,7 +1067,7 @@ void FieldTrialList::InstantiateFieldTrialAllocatorIfNeeded() {
AddToAllocatorWhileLocked(registered.second);
}
-#if defined(OS_WIN)
+#if defined(OS_WIN) || defined(POSIX_WITH_ZYGOTE)
// Set |readonly_allocator_handle_| so we can pass it to be inherited and
// via the command line.
global_->readonly_allocator_handle_ =
@@ -782,25 +813,33 @@ void FieldTrialList::CopyFieldTrialStateToFlags(
return;
}
- // HANDLE is just typedef'd to void *. We basically cast the handle into an
- // int (uintptr_t, to be exact), stringify the int, and pass it as a
- // command-line flag. The child process will do the reverse conversions to
- // retrieve the handle. See http://stackoverflow.com/a/153077
+#if defined(OS_WIN)
+ // PlatformFile is typedef'd to HANDLE which is typedef'd to void *. We
+ // basically cast the handle into an int (uintptr_t, to be exact), stringify
+ // the int, and pass it as a command-line flag. The child process will do
+ // the reverse conversions to retrieve the handle. See
+ // http://stackoverflow.com/a/153077
auto uintptr_handle =
reinterpret_cast<uintptr_t>(global_->readonly_allocator_handle_);
« no previous file with comments | « base/metrics/field_trial.h ('k') | base/metrics/field_trial_unittest.cc » ('j') | no next file with comments »

Powered by Google App Engine
This is Rietveld 408576698